﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Exam.Infrastructure.Utilities
{
    public class BaseAssemblyUtility
    {
        public static string ProjectPrefix = @"Exam";

        public static string API = string.Format(@"^{0}\.(.*\.)?Api$", ProjectPrefix);

        public static string CLIENT = @"^PlanReady\.(.*\.)?Client";

        public static string EXTENSIONS = @"^PlanReady\.Extensions\..*$";

        public static string REPOSITORY = string.Format(@"^{0}\.(.*\.)?Repository", ProjectPrefix);

        public static string DOMAIN = string.Format(@"^{0}\.(.*\.)?Domain", ProjectPrefix);

        public static string CONSTRACT = string.Format(@"^{0}\.(.*\.)?Contract", ProjectPrefix);

        public static string APISERVICE = string.Format(@"^{0}\.(.*\.)?ApiService", ProjectPrefix);

        public static string VITRUALDOMAIN = string.Format(@"^{0}\.(.*\.)?Domain", ProjectPrefix);

        public static string VITRUALCONSTRACT = string.Format(@"^{0}\.(.*\.)?Contract", ProjectPrefix);

        public static string VITRUALAPISERVICE = string.Format(@"^{0}\.(.*\.)?ApiService", ProjectPrefix);

        public static string VITRUALREPOSITORY = string.Format(@"^{0}\.(.*\.)?Repository", ProjectPrefix);

        public static string VITRUALSIGNATURE = string.Format(@"^{0}\.(.*\.)?Signature", ProjectPrefix);

        public static string VITRUALENTITY = string.Format(@"{0}", ProjectPrefix);

        public static string VITRUALSHARE = string.Format(@"^{0}\.(.*\.)?Share", ProjectPrefix);

        public static string VITRUALAPPLICATION = string.Format(@"^{0}\.(.*\.)?Application", ProjectPrefix);

        public static string ALL = "*.dll";



        static BaseAssemblyUtility()
        {
            //Hashtable = new Hashtable();
        }

        public static IServiceProvider Provider { get; set; }

        private static string[] FinderFilter
        {
            get
            {
                return new string[] { API, CLIENT, EXTENSIONS };
            }
        }
        private static string[] ModuleFilter
        {
            get
            {
                return new string[] { @"^PlanReady\.(.*\.)?Services$", @"^PlanReady\.(.*\.)\.Data$" };
            }
        }

        public static string TraceId
        {
            get
            {
                try
                {
                    var httpContextAccessor = Provider.GetService<IHttpContextAccessor>();

                    if (httpContextAccessor == null) return null;

                    var id = httpContextAccessor.HttpContext?.TraceIdentifier ?? "defaultId";

                    return id;
                }
                catch
                {
                    return null;
                }
            }
        }

        //public static Hashtable Hashtable { get; set; }

        public static List<Assembly> GetAssembly(string path)
        {
            //dynamic type = (new Program()).GetType();
            string currentDirectory = Path.GetDirectoryName(path);
            var files = Directory.GetFiles(currentDirectory, ALL);
            var assemblys = new List<Assembly>();

            foreach (var file in files)
            {
                assemblys.Add(Assembly.LoadFrom(file));
            }

            return assemblys;
        }

        public static List<Assembly> CreateModules(List<Assembly> assemblies)
        {
            List<Assembly> modules = new List<Assembly>();
            foreach (var filter in ModuleFilter)
            {
                CreateModulesByFilter(assemblies, filter);
            }
            return modules;
        }

        public static List<Assembly> CreateModulesByFilter(List<Assembly> assemblies, string filter)
        {
            List<Assembly> modules = new List<Assembly>();
            modules.AddRange(
                assemblies.Where(item => Regex.IsMatch(Path.GetFileNameWithoutExtension(item.CodeBase), filter)));
            return modules;
        }

        public static Dictionary<string, List<Assembly>> CreateFinders(List<Assembly> assemblies)
        {
            Dictionary<string, List<Assembly>> finders = new Dictionary<string, List<Assembly>>();
            foreach (var filter in FinderFilter)
            {
                if (!finders.ContainsKey(filter))
                {
                    finders[filter] = new List<Assembly>();
                }
                finders[filter]
                    .AddRange(assemblies.Where(item => Regex.IsMatch(Path.GetFileNameWithoutExtension(item.CodeBase), filter)));
            }
            return finders;
        }



        public static Tuple<Dictionary<string, List<Assembly>>, Assembly[]> ScannerAssemblies(string path)
        {
            var assemblies = GetAssembly(path);

            var finders = CreateFinders(assemblies);

            var modules = CreateModules(assemblies);

            return new Tuple<Dictionary<string, List<Assembly>>, Assembly[]>(finders, modules.ToArray());
        }

        public static List<Type> GetTypeToRegister<TEntity>(List<Assembly> assemblies, string filter)
        {
            var types = new List<Type>();

            var currentDomainAssemblies = CreateModulesByFilter(assemblies, filter);


            foreach (var currentAssembly in currentDomainAssemblies)
            {
                types.AddRange(currentAssembly.GetTypes().Where(q => q.GetInterface(typeof(TEntity).FullName) != null));

            }

            return types;
        }

        public static void RegisterInstance(IServiceCollection container, List<Assembly> assemblies, List<Type> interfaces, string filter)
        {

            var currentDomainAssemblies = CreateModulesByFilter(assemblies, filter);

            foreach (var currentAssembly in currentDomainAssemblies)
            {
                foreach (var item in interfaces)
                {
                    var types = currentAssembly.GetTypes().Where(m => m.GetInterfaces().Contains(item) && !m.IsInterface).ToList();

                    foreach (var type in types)
                    {
                        container.AddSingleton(type.GetInterfaces().FirstOrDefault(m => m != item), type);
                        //instatnces.Add((TEntity)currentAssembly.CreateInstance(type.FullName));
                    }

                }
            }
        }

        public static void RegisterAddTransientInstance(IServiceCollection container, List<Assembly> assemblies, List<Type> interfaces, string filter)
        {

            var currentDomainAssemblies = CreateModulesByFilter(assemblies, filter);

            foreach (var currentAssembly in currentDomainAssemblies)
            {
                foreach (var item in interfaces)
                {
                    var types = currentAssembly.GetTypes().Where(m => m.GetInterfaces().Contains(item) && !m.IsInterface).ToList();

                    foreach (var type in types)
                    {
                        container.AddTransient(type.GetInterfaces().FirstOrDefault(m => m != item), type);
                        //instatnces.Add((TEntity)currentAssembly.CreateInstance(type.FullName));
                    }

                }
            }
        }

        public static List<TEntity> CreateInstance<TEntity>(string filter)
        {
            var instatnces = new List<TEntity>();

            instatnces = Provider.GetServices<TEntity>().Where(item => Regex.IsMatch(Path.GetFileNameWithoutExtension(item.GetType().FullName), filter)).ToList();

            return instatnces;
        }

        public static List<TEntity> CreateInstance<TEntity>(List<Assembly> assemblies, string filter)
        {
            var instatnces = new List<TEntity>();

            var currentDomainAssemblies = CreateModulesByFilter(assemblies, filter);

            foreach (var currentAssembly in currentDomainAssemblies)
            {
                var types = currentAssembly.GetTypes().Where(q => q.GetInterface(typeof(TEntity).FullName) != null);

                foreach (var type in types)
                {
                    instatnces.Add((TEntity)currentAssembly.CreateInstance(type.FullName));
                }
            }

            return instatnces;
        }
    }
}
