﻿using Exam.Infrastructure.Data.Entity;
using FluentValidation;

namespace Exam.Insfrastructure.Service.Interface
{
    public interface IExamRepository<TEntity, TDBContext> :IRepository<TEntity> where TEntity : class, IEntity<string>, IHasCreationTime, IDataPermission, new()
    where TDBContext: SugarUnitOfWork, new()
    {
        public ISugarUnitOfWork<TDBContext> UOW { get; }

        public AbstractValidator<TEntity> Validator { get; set; }

        /// <summary>
        /// 单表新增
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public Task AddWithValidateAsync(TEntity entity, CancellationToken cancellationToken);

        /// <summary>
        /// 单表批量新增
        /// </summary>
        /// <param name="entities"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public Task AddWithValidateAsync(List<TEntity> entities, CancellationToken cancellationToken);

        /// <summary>
        /// 单表删除
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public Task DeleteWithValidateAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken);

        /// <summary>
        /// 单表修改
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public Task UpdateWithValidateAsync(TEntity entity, CancellationToken cancellationToken);

        /// <summary>
        /// 单表修改
        /// </summary>
        /// <param name="entities"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public Task UpdateWithValidateAsync(List<TEntity> entities, CancellationToken cancellationToken);

        
    }
}
