using Exam.Application;
using Exam.Infrastructure.Data.Entity;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Share;
using Hotline.Application.Exam.QueryExtensions.Questions;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Questions;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Requests.Question;
using Mapster;
using System.ComponentModel;
using XF.Domain.Dependency;

namespace Hotline.Application.Exam.Service.Questions
{
    /// <summary>
    /// 试题参考答案服务
    /// </summary>
    [Description("试题参考答案服务")]
    public class QuestionAnswerService : ApiService<QuestionAnswer, QuestionAnswerDto,HotlineDbContext>,IQuestionAnswerService, IScopeDependency
    {
        private readonly IQuestionAnswerRepository _repository;
        public QuestionAnswerService(IQuestionAnswerRepository repository) : base(repository)
        {
            _repository = repository;
        }

        public async Task<QuestionAnswerDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            return entity.Adapt<QuestionAnswerDto>();
        }

        public async Task<(int, List<QuestionAnswerViewResponse>)> GetListAsync(QuestionAnswerPagedRequest queryRequest)
        {
            var query = _repository.Queryable();

            var result = await query.Select(m => new QuestionAnswerViewResponse { 
                				QuestionId = m.QuestionId,
								Answer = m.Answer,
								Id = m.Id,
				            }).ToListAsync();

            var total = await query.CountAsync();

            return (total,result);
        }

        public async Task<PageViewResponse<QuestionAnswerViewResponse>> GetPagedListAsync(QuestionAnswerPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionAnswerTable = _repository.Queryable().Where(expression);

            var queryable = questionAnswerTable.Select((m) => new QuestionAnswerViewResponse
            {
                				QuestionId = m.QuestionId,
								Answer = m.Answer,
								Id = m.Id,
				            });

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new PageViewResponse<QuestionAnswerViewResponse>
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }
    }
}


