using Exam.Application;
using Exam.Infrastructure.Data.Entity;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Share;
using Hotline.Application.Exam.QueryExtensions.Questions;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Questions;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Requests.Question;
using Mapster;
using System.ComponentModel;
using XF.Domain.Dependency;

namespace Hotline.Application.Exam.Service.Questions
{
    /// <summary>
    /// 试题选项服务
    /// </summary>
    [Description("试题选项服务")]
    public class QuestionOptionsService : ApiService<QuestionOptions, QuestionOptionsDto,HotlineDbContext>,IQuestionOptionsService, IScopeDependency
    {
        private readonly IQuestionOptionsRepository _repository;
        public QuestionOptionsService(IQuestionOptionsRepository repository) : base(repository)
        {
            _repository = repository;
        }

        public async Task<QuestionOptionsDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            return entity.Adapt<QuestionOptionsDto>();
        }

        public async Task<(int, List<QuestionOptionsViewResponse>)> GetListAsync(QuestionOptionsPagedRequest queryRequest)
        {
            var query = _repository.Queryable();

            var result = await query.Select(m => new QuestionOptionsViewResponse { 
                				Content = m.Content,
								IsAnswer = m.IsAnswer,
								QuestionId = m.QuestionId,
								Id = m.Id,
				            }).ToListAsync();

            var total = await query.CountAsync();

            return (total,result);
        }

        public async Task<PageViewResponse<QuestionOptionsViewResponse>> GetPagedListAsync(QuestionOptionsPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionOptionsTable = _repository.Queryable().Where(expression);

            var queryable = questionOptionsTable.Select((m) => new QuestionOptionsViewResponse
            {
                				Content = m.Content,
								IsAnswer = m.IsAnswer,
								QuestionId = m.QuestionId,
								Id = m.Id,
				            });

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new PageViewResponse<QuestionOptionsViewResponse>
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }
    }
}


