using Exam.Infrastructure.Data.Entity;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Share;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Questions;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Requests.Question;
using Mapster;
using System.ComponentModel;
using XF.Domain.Dependency;

namespace Exam.Application
{
    /// <summary>
    /// 试题标签服务
    /// </summary>
    [Description("试题标签服务")]
    public class QuestionTagService : ApiService<QuestionTag, QuestionTagDto,HotlineDbContext>,IQuestionTagService, IScopeDependency
    {
        private readonly IQuestionTagRepository _repository;
        public QuestionTagService(IQuestionTagRepository repository) : base(repository)
        {
            _repository = repository;
        }

        public async Task<QuestionTagDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            return entity.Adapt<QuestionTagDto>();
        }

        public async Task<(int, List<QuestionTagViewResponse>)> GetListAsync(QuestionTagPagedRequest queryRequest)
        {
            var query = _repository.Queryable();

            var result = await query.Select(m => new QuestionTagViewResponse { 
                				TagId = m.TagId,
								QuestionId = m.QuestionId,
								Id = m.Id,
				            }).ToListAsync();

            var total = await query.CountAsync();

            return (total,result);
        }

        public async Task<PageViewResponse<QuestionTagViewResponse>> GetPagedListAsync(QuestionTagPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionTagTable = _repository.Queryable().Where(expression);

            var queryable = questionTagTable.Select((m) => new QuestionTagViewResponse
            {
                				TagId = m.TagId,
								QuestionId = m.QuestionId,
								Id = m.Id,
				            });

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new PageViewResponse<QuestionTagViewResponse>
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }
    }
}


